#!/usr/bin/env python

import math, urllib
import graphics
import sys
 
class GoogleMaps:
    def __init__(self, api_key):
        self.api_key = api_key
        self.offset = 268435456
        self.radius = self.offset / math.pi
        self.tmp_file = (sys.path[0] + "/glade/map.png")
 
    def geocodeAddress(self, address):
        res = urllib.urlopen(self.getGeocodeUrl(address)).read()
        data = res.split(',')
        if data[0] != "200":
            errorCode = int(data[0])
            raise Exception("Google Maps Exception: %s" %
                            self.getGeocodeError(errorCode))
        else:
            # In csv format the information retrieved is as follows:
            # 1. HTTP status code, (index 0 in python lists).
            # 2. accuracy (See accuracy constants) (index 1 in python lists).
            # 3. latitude (index 2 in python lists).
            # 4. longitude (index 3 in python lists).
            return [float(data[2]),float(data[3])]
 
    def retrieveStaticImage(self, width, height, lng, lat, zoom, format):
        urllib.urlretrieve(self.getMapUrl(width, height,
                                          lng, lat, zoom,
                                          format), self.tmp_file)
        if sys.platform=='linux':
            tmp_file = open(self.tmp_file)
        else:
            tmp_file = open(self.tmp_file)
        return tmp_file
 
    def getGeocodeError(self, errorCode):
        dictError = {400: "Bad request",
                     500: "Server error",
                     601: "Missing query",
                     602: "Unknown address",
                     603: "Unavailable address",
                     604: "Unknown directions",
                     610: "Bad API key",
                     620: "Too many queries"}
 
        if dictError.has_key(errorCode):
            return dictError[errorCode]
        else:
            return "Generic error"
 
    def getGeocodeUrl(self, url):
        urlbase = "http://maps.google.com/maps/geo"
        return "%(urlbase)s?%(params)s" % {'params': urllib.urlencode({'q':url,
                                         'output':'csv',
                                         'key':self.api_key
                                        }),
             'urlbase': urlbase}
 
    def getMapUrl(self, width, height, lng, lat, zoom, format):
        urlbase = "http://maps.google.com/staticmap"
        params = ["center=%(lat)s,%(lng)s" % {"lat":lat,"lng":lng}]
        params.append("format=%(format)s" % {"format":format})
        params.append("zoom=%(zoom)s" % {"zoom":zoom})
        params.append("size=%(width)sx%(height)s" % {"width":width,"height":height})
        params.append("key=%(api_key)s" % {"api_key":self.api_key})
        return  "%(urlbase)s?%(params)s" % {"urlbase":urlbase,"params":"&".join(params)}
 
    def adjust(self, lat, lng, deltaX, deltaY, z):
        return (self.XToL( self.LToX(lat) + (deltaX<<(21-z))),
               self.YToL(self.LToY(lng) + (deltaY<<(21-z))))
 
    def LToX(self, x):
        return round(self.offset + self.radius * x * math.pi / 180)
 
    def LToY(self, y):
        return round( self.offset - self.radius *
            (math.log((
            (1 + math.sin(y * math.pi / 180))
            /
            (1 - math.sin(y * math.pi / 180))
            )
        )) / 2)
 
    def XToL(self, x):
        return ((round(x) - self.offset) / self.radius) * 180 / math.pi
 
    def YToL(self, y):
        return (math.pi / 2 - 2 * (
                math.atan(
                    math.exp(((round(y)-self.offset)/self.radius))
                )
            )) * 180 / math.pi
